
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "qpid/agent/ManagementAgent.h"
#include "Domain.h"
#include "ArgsDomainSave.h"
#include "ArgsDomainRestore.h"
#include "ArgsDomainGetXMLDesc.h"
#include "ArgsDomainMigrate.h"


using namespace qmf::com::redhat::libvirt;
using namespace qpid::framing;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           std::string;

string  Domain::packageName  = string ("com.redhat.libvirt");
string  Domain::className    = string ("domain");
uint8_t Domain::md5Sum[16]   =
    {0xb8,0x4f,0x5e,0xed,0x5e,0xd5,0x3b,0x70,0x7,0x7b,0x12,0x7b,0x3f,0x3e,0x5f,0xa5};

Domain::Domain (ManagementAgent*, Manageable* _core, ::qpid::management::Manageable* _parent, const std::string& _uuid, const std::string& _name) :
    ManagementObject(_core),uuid(_uuid),name(_name)
{
    node = _parent->GetManagementObject ()->getObjectId ();
    id = 0;
    state = "";
    numVcpus = 0;
    active = "";
    maximumMemory = 0;
    memory = 0;
    cpuTime = 0;



}

Domain::~Domain ()
{

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Domain::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Domain::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putOctet       (0);           // No Superclass
    buf.putShort       (7); // Config Element Count
    buf.putShort       (3); // Inst Element Count
    buf.putShort       (11); // Method Count

    // Properties
    ft.clear();
    ft.setString (NAME, "uuid");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Domain UUID");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "name");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Domain name");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "id");
    ft.setInt    (TYPE, TYPE_S64);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Hypervisor Domain id, -1 if not running.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "node");
    ft.setInt    (TYPE, TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 1);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Node of this VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "state");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Domain state.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "numVcpus");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Number of virtual CPUs.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "active");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "true/false domain active.");
    buf.put (ft);


    // Statistics
    ft.clear();
    ft.setString (NAME,   "maximumMemory");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Maximum memory allowed in KB.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "memory");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Memory used in KB.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "cpuTime");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "CPU time used in nanoseconds.");
    buf.put (ft);


    // Methods
    ft.clear();
    ft.setString (NAME,     "create");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Start stopped VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "destroy");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Stop this VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "undefine");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Undefine the domain");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "suspend");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Suspend this VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "resume");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Resume suspended VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "save");
    ft.setInt    (ARGCOUNT, 1);
    ft.setString (DESC,     "Save this VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "filename");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Name of file to save VM image in");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "restore");
    ft.setInt    (ARGCOUNT, 1);
    ft.setString (DESC,     "Restore saved VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "filename");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Name of file to restore from");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "shutdown");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Shutdown a VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "reboot");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Reboot VM");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "getXMLDesc");
    ft.setInt    (ARGCOUNT, 1);
    ft.setString (DESC,     "Return an XML description of the domain.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "description");
    ft.setInt    (TYPE,    TYPE_LSTR);
    ft.setString (DIR,     "O");
    ft.setString (DESC,    "XML description of the domain.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "migrate");
    ft.setInt    (ARGCOUNT, 5);
    ft.setString (DESC,     "Migrate this VM to another host");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "destinationUri");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Destination host connection uri.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "flags");
    ft.setInt    (TYPE,    TYPE_U32);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Flags for migration, see documentation for details.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "newDomainName");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "New domain name on new host if applicable.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "uri");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Destination hostname/URI as seen from host.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "bandwidth");
    ft.setInt    (TYPE,    TYPE_U32);
    ft.setString (DIR,     "I");
    ft.setString (DESC,    "Maximum bandwidth to consume in Mbps.");
    buf.put (ft);


}



void Domain::writeProperties (Buffer& buf)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);

    buf.putShortString(uuid);
    buf.putShortString(name);
    buf.putInt64(id);
    node.encode(buf);
    buf.putShortString(state);
    buf.putLong(numVcpus);
    buf.putShortString(active);

}

void Domain::writeStatistics (Buffer& buf, bool skipHeaders)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;



    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putLongLong(maximumMemory);
    buf.putLongLong(memory);
    buf.putLongLong(cpuTime);


    // Maintenance of hi-lo statistics


}

void Domain::doMethod (string& methodName, Buffer& inBuf, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    if (methodName == "create") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_CREATE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "destroy") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_DESTROY, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "undefine") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_UNDEFINE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "suspend") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_SUSPEND, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "resume") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_RESUME, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "save") {
        ArgsDomainSave ioArgs;
        inBuf.getShortString(ioArgs.i_filename);
        status = coreObject->ManagementMethod (METHOD_SAVE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "restore") {
        ArgsDomainRestore ioArgs;
        inBuf.getShortString(ioArgs.i_filename);
        status = coreObject->ManagementMethod (METHOD_RESTORE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "shutdown") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_SHUTDOWN, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "reboot") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_REBOOT, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    if (methodName == "getXMLDesc") {
        ArgsDomainGetXMLDesc ioArgs;
        status = coreObject->ManagementMethod (METHOD_GETXMLDESC, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        outBuf.putMediumString(ioArgs.o_description);
        return;
    }

    if (methodName == "migrate") {
        ArgsDomainMigrate ioArgs;
        inBuf.getShortString(ioArgs.i_destinationUri);
        ioArgs.i_flags = inBuf.getLong();
        inBuf.getShortString(ioArgs.i_newDomainName);
        inBuf.getShortString(ioArgs.i_uri);
        ioArgs.i_bandwidth = inBuf.getLong();
        status = coreObject->ManagementMethod (METHOD_MIGRATE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    outBuf.putLong(status);
    outBuf.putShortString(Manageable::StatusText(status, text));
}
