
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "qpid/agent/ManagementAgent.h"
#include "Volume.h"
#include "ArgsVolumeGetXMLDesc.h"


using namespace qmf::com::redhat::libvirt;
using namespace qpid::framing;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           std::string;

string  Volume::packageName  = string ("com.redhat.libvirt");
string  Volume::className    = string ("volume");
uint8_t Volume::md5Sum[16]   =
    {0x91,0x2b,0x54,0x94,0x99,0x55,0x5f,0xcd,0x30,0xc2,0x28,0xdd,0xb6,0x66,0xcd,0xb9};

Volume::Volume (ManagementAgent*, Manageable* _core, ::qpid::management::Manageable* _parent, const std::string& _key, const std::string& _path, const std::string& _name, const std::string& _childLVMName) :
    ManagementObject(_core),key(_key),path(_path),name(_name),childLVMName(_childLVMName)
{
    storagePool = _parent->GetManagementObject ()->getObjectId ();
    capacity = 0;
    allocation = 0;



}

Volume::~Volume ()
{

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Volume::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Volume::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putOctet       (0);           // No Superclass
    buf.putShort       (7); // Config Element Count
    buf.putShort       (0); // Inst Element Count
    buf.putShort       (2); // Method Count

    // Properties
    ft.clear();
    ft.setString (NAME, "key");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Storage volume key");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "path");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Storage volume path");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "name");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Storage volume name");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "childLVMName");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "The name of the LVM pool associated with this volume, or blank of non-existant.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "capacity");
    ft.setInt    (TYPE, TYPE_U64);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Capacity of storage volume in bytes");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "allocation");
    ft.setInt    (TYPE, TYPE_U64);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Current allocation in bytes");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "storagePool");
    ft.setInt    (TYPE, TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (IS_INDEX, 1);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Pool where this storage volume exists");
    buf.put (ft);


    // Statistics

    // Methods
    ft.clear();
    ft.setString (NAME,     "getXMLDesc");
    ft.setInt    (ARGCOUNT, 1);
    ft.setString (DESC,     "Return an XML description of the volume.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,    "description");
    ft.setInt    (TYPE,    TYPE_LSTR);
    ft.setString (DIR,     "O");
    ft.setString (DESC,    "XML description of the volume.");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,     "delete");
    ft.setInt    (ARGCOUNT, 0);
    ft.setString (DESC,     "Delete the storage volume from the pool.");
    buf.put (ft);


}



void Volume::writeProperties (Buffer& buf)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);

    buf.putShortString(key);
    buf.putShortString(path);
    buf.putShortString(name);
    buf.putShortString(childLVMName);
    buf.putLongLong(capacity);
    buf.putLongLong(allocation);
    storagePool.encode(buf);

}

void Volume::writeStatistics (Buffer& buf, bool skipHeaders)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;



    if (!skipHeaders)
        writeTimestamps (buf);


    // Maintenance of hi-lo statistics


}

void Volume::doMethod (string& methodName, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    if (methodName == "getXMLDesc") {
        ArgsVolumeGetXMLDesc ioArgs;
        status = coreObject->ManagementMethod (METHOD_GETXMLDESC, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        outBuf.putMediumString(ioArgs.o_description);
        return;
    }

    if (methodName == "delete") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_DELETE, ioArgs, text);
        outBuf.putLong        (status);
        outBuf.putMediumString(::qpid::management::Manageable::StatusText (status, text));
        return;
    }

    outBuf.putLong(status);
    outBuf.putShortString(Manageable::StatusText(status, text));
}
